import { ChangeEvent, Component, ReactNode } from 'react';
/**
 * A predicate function used for filtering items.
 *
 * @example
 * ```typescript
 *
 * interface User {
 *   name: string;
 *   email: string;
 * };
 *
 * const predicate: IFilteringPredicate<User> = (user, value) =>
 *   user.name.includes(value) || user.email.includes(value);
 * ```
 *
 * @typeparam T - The type of items that will be filtered using the predicate.
 * @param item - The current item in the filtering process.
 * @param value - A search value used for filtering.
 */
export interface IFilteringPredicate<T> {
    (item: T, value: string): boolean;
}
export interface IRenderProp<T> {
    (props: {
        items: T[];
        value: string;
        handleChange(event: ChangeEvent<HTMLInputElement>): void;
    }): ReactNode;
}
interface IBaseProps<T> {
    /**
     * Should the filtering function be debounced?
     */
    debounce: boolean;
    /**
     * The amount in ms to debounce the filtering method.
     */
    debounceDuration: number;
    /**
     * An initial search value. Will affect initial state.
     */
    initialValue: string;
    /**
     * The array of items to filter.
     */
    items: T[];
    /**
     * The predicate used for filtering items.
     */
    predicate: IFilteringPredicate<T>;
    children: IRenderProp<T>;
    render: IRenderProp<T>;
}
interface IPropsWithChildren<T> extends IBaseProps<T> {
    children: IRenderProp<T>;
}
interface IPropsWithRender<T> extends IBaseProps<T> {
    render: IRenderProp<T>;
}
export declare type IProps<T> = IPropsWithChildren<T> | IPropsWithRender<T>;
interface IState<T> {
    /**
     * An array of filtered items based on [[IProps.items]].
     */
    items: T[];
    /**
     * Current value used for filtering.
     */
    value: string;
}
/**
 * @typeparam T - The type of items to search.
 */
export default class Searchable<T> extends Component<IProps<T>, IState<T>> {
    static defaultProps: {
        debounce: boolean;
        debounceDuration: number;
        initialValue: string;
    };
    /**
     * Filters an array of items based on a search value and a filtering predicate.
     *
     * @typeparam T - The type of items.
     * @param items - The array of items to be filtered.
     * @param value - A search string to be passed to the predicate.
     * @param predicate - A filtering predicate based on an item and the search value.
     * @returns A filtered array of items.
     */
    static filter<T>(items: T[], value: string, predicate: IFilteringPredicate<T>): T[];
    constructor(props: IProps<T>);
    /**
     * Change event handler for updating value in state.
     *
     * @param event - An instance of ChangeEvent
     */
    handleChange: (event: ChangeEvent<HTMLInputElement>) => void;
    /**
     * Call filtering method if [[IState.value]] has changed.
     */
    componentDidUpdate(prevProps: IProps<T>, prevState: IState<T>): void;
    render(): ReactNode;
    /**
     * Filters an array of items and sets state. This method might be debounced in [[constructor]].
     *
     * @param items - The array of items to filter.
     * @param value - The search value to base the filtering on.
     */
    filterAndSetState(items: T[], value: string): void;
}
export {};
